@@ -1,10 +1,12 @@
 import base64
+import json
 
-import simplejson
 from eth_account.messages import defunct_hash_message
 from web3.auto import w3
 
-from magic_admin.error import DIDTokenError
+from magic_admin.error import DIDTokenExpired
+from magic_admin.error import DIDTokenInvalid
+from magic_admin.error import DIDTokenMalformed
 from magic_admin.resources.base import ResourceComponent
 from magic_admin.utils.did_token import parse_public_address_from_issuer
 from magic_admin.utils.time import apply_did_token_nbf_grace_period
@@ -42,7 +44,7 @@ class Token(ResourceComponent):
                 missing_fields.append(field)
 
         if missing_fields:
-            raise DIDTokenError(
+            raise DIDTokenMalformed(
                 message='DID token is missing required field(s): {}'.format(
                     sorted(missing_fields),
                 ),
@@ -55,18 +57,18 @@ class Token(ResourceComponent):
             did_token (base64.str): Base64 encoded string.
 
         Raises:
-            DIDTokenError: If token format is invalid.
+            DIDTokenMalformed: If token format is invalid.
 
         Returns:
             proof (str): A signed message.
             claim (dict): A dict of unsigned message.
         """
         try:
-            decoded_did_token = simplejson.loads(
+            decoded_did_token = json.loads(
                 base64.urlsafe_b64decode(did_token).decode('utf-8'),
             )
         except Exception as e:
-            raise DIDTokenError(
+            raise DIDTokenMalformed(
                 message='DID token is malformed. It has to be a based64 encoded '
                 'JSON serialized string. {err} ({msg}).'.format(
                     err=e.__class__.__name__,
@@ -75,7 +77,7 @@ class Token(ResourceComponent):
             )
 
         if len(decoded_did_token) != EXPECTED_DID_TOKEN_CONTENT_LENGTH:
-            raise DIDTokenError(
+            raise DIDTokenMalformed(
                 message='DID token is malformed. It has to have two parts '
                 '[proof, claim].',
             )
@@ -83,9 +85,9 @@ class Token(ResourceComponent):
         proof = decoded_did_token[0]
 
         try:
-            claim = simplejson.loads(decoded_did_token[1])
+            claim = json.loads(decoded_did_token[1])
         except Exception as e:
-            raise DIDTokenError(
+            raise DIDTokenMalformed(
                 message='DID token is malformed. Given claim should be a JSON '
                 'serialized string. {err} ({msg}).'.format(
                     err=e.__class__.__name__,
@@ -130,21 +132,29 @@ class Token(ResourceComponent):
             did_token (base64.str): Base64 encoded string.
 
         Raises:
-            DIDTokenError: If DID token fails the validation.
+            DIDTokenInvalid: If DID token fails the validation.
+            DIDTokenExpired: If DID token has expired.
 
         Returns:
             None.
         """
         proof, claim = cls.decode(did_token)
+
+        if claim['ext'] is None:
+            raise DIDTokenInvalid(
+                message='Please check the "ext" field and regenerate a new token '
+                'with a suitable value.',
+            )
+
         recovered_address = w3.eth.account.recoverHash(
             defunct_hash_message(
-                text=simplejson.dumps(claim, separators=(',', ':')),
+                text=json.dumps(claim, separators=(',', ':')),
             ),
             signature=proof,
         )
 
         if recovered_address != cls.get_public_address(did_token):
-            raise DIDTokenError(
+            raise DIDTokenInvalid(
                 message='Signature mismatch between "proof" and "claim". Please '
                 'generate a new token with an intended issuer.',
             )
@@ -152,12 +162,12 @@ class Token(ResourceComponent):
         current_time_in_s = epoch_time_now()
 
         if current_time_in_s > claim['ext']:
-            raise DIDTokenError(
+            raise DIDTokenExpired(
                 message='Given DID token has expired. Please generate a new one.',
             )
 
         if current_time_in_s < apply_did_token_nbf_grace_period(claim['nbf']):
-            raise DIDTokenError(
+            raise DIDTokenInvalid(
                 message='Given DID token cannot be used at this time. Please '
                 'check the "nbf" field and regenerate a new token with a suitable '
                 'value.',
